#!/usr/bin/perl
#
# makeISOGraph.pl
#
# Reads in graph information and produces a project chart

# Default parameters
$colormap="colormap.txt";
$nodenames="objectnames.txt";
$relations="relations.txt";
$shapes="shapemap.txt";
$status="status.txt";
$graphAttributes="UNSET";
$defaultColor="yellow";
$defaultShape="invtriangle";
$objectiveLen="1.0";
$areaLen="0.1";
$areaFont="72";
$rootFont="128";
$objectiveFont="24";
$areaWeight=5;

# Parse command-line parameters
$numargs=@ARGV;
        foreach $argument (@ARGV)
        {
		chomp($argument);
                @pair=split(/\:/,$argument);
                if (@pair[0] eq "cm")
                {
                        $colormap=@pair[1];
                } elsif (@pair[0] eq "sm")
                {
                        $shapes=@pair[1];
                } elsif (@pair[0] eq "on")
                {
                        $nodenames=@pair[1];
                } elsif (@pair[0] eq "st")
                {
                        $status=@pair[1];
                } elsif (@pair[0] eq "ga")
                {
                        $graphAttributes=@pair[1];
                } elsif (@pair[0] eq "af")
                {
                        $areaFont=@pair[1];
		} elsif (@pair[0] eq "of")
		{
			$objectiveFont=@pair[1];
		} elsif (@pair[0] eq "rf")
		{
			$rootFont=@pair[1];
		} elsif (@pair[0] eq "rm")
		{
			$relations=@pair[1];
                } else{
                        print "@pair[0] is an unsupported option\n";
                        &showUsage;
                        die;
                }
        } # (foreach)

# Sanity check


# Say hello
print "/* This .dot output was generated by makeISOGraph.pl /*\n";
print "/*    by Foofus.\n";
print "/*    *** NOTE: If you see $defaultColor nodes in the shape of $defaultShape, */\n";
print "/*    ***       then something has gone wrong.  Check your status file... */\n";

# Read in color map, assign ordered values
open(INFILE,"<$colormap") || die "ERROR: Could not open $colormap for reading!\n";
@colors=<INFILE>;
close(INFILE);
print "/* Read in color mappings from $colormap */\n";
%colorHash=("nodeStatus" => "nodeColor"); 
%statusHash=("nodeStatus" => "statusValue");
$value=0;
foreach $color (@colors)
{
	chomp($color);
	@elements=split(/:/,$color);
	$colorHash{@elements[0]}=@elements[1];
	$statusHash{@elements[0]}=$value;
	$value++;
} # foreach

# Read in shape list
open(INFILE,"<$shapes") || die "ERROR: Could not open $shapes for reading!\n";
@shapes=<INFILE>;
close(INFILE);
print "/* Read in node shapes from $shapes */\n";
%shapeMapHash=("nodeType" => "nodeShape"); 
foreach $shape (@shapes)
{
	chomp($shape);
	@elements=split(/:/,$shape);
	$shapeMapHash{@elements[0]}=@elements[1];
} # foreach


# Read in graph nodes
open(INFILE,"<$nodenames") || die "ERROR: Could not open $nodenames for reading!\n";
@nodes=<INFILE>;
close(INFILE);
print "/* Read in node names from $nodenames */\n";
%shapeTypeHash=("nodeName" => "nodeType"); 
foreach $node (@nodes)
{
	chomp($node);
	@elements=split(/:/,$node);
	$shapeTypeHash{@elements[0]}=@elements[1];
} # foreach

# Read in graph structure
open(INFILE,"<$relations") || die "ERROR: Could not open $relations for reading!\n";
@relations=<INFILE>;
close(INFILE);
print "/* Read in node relationships from $relations */\n";
%nodeParentHash=("nodeName" => "nodeParent"); 
foreach $node (@relations)
{
	chomp($node);
	@elements=split(/:/,$node);
	$nodeParentHash{@elements[0]}=@elements[1];
} # foreach

# Read in project status
open(INFILE,"<$status") || die "ERROR: Could not open $status for reading!\n";
@status=<INFILE>;
close(INFILE);
print "/* Read in project status from $status */\n";

# Make graph header
print "digraph G {\n";
if ($graphAttributes ne "UNSET")
{
	print "  $graphAttributes\n";
}

# For each node, assign shape and color based on map and status
# Do objectives first, and calculate parent status
%parentStatus=("nodeName" => "nodeStatus");
print "  /* Describing objective nodes */ \n";
foreach $node (@status)
{
	# Set up for this pass (if you see these attributes, something is wrong!)
	$color=$defaultColor;
	$shape=$defaultShape;
	chomp($node);
	$nodeName="";
	$nodeStatus="";
	$type="";

	# Gather information
	@elements=split(/:/,$node);
	$nodeName=@elements[0];
	$nodeStatus=@elements[1];
	$type=$shapeTypeHash{$nodeName};
	if ($type eq "OBJECTIVE")
	{
		$color=$colorHash{$nodeStatus};
		$shape=$shapeMapHash{$shapeTypeHash{$nodeName}};
		$label=$nodeName;
		$label=~s/_/\\n/g;
		print "  $nodeName [label=\"$label\",shape=$shape,color=$color,style=filled,fontsize=$objectiveFont];\n";

		# Adjust parent's status, if needed 
		if (exists $parentStatus{$nodeParentHash{$nodeName}})
		{
			if (($statusHash{$parentStatus{$nodeParentHash{$nodeName}}} > $statusHash{$nodeStatus})and($statusHash{$nodeStatus} ne "0"))
			{
				$parentStatus{$nodeParentHash{$nodeName}} = $nodeStatus;
			}
		} else {
			$parentStatus{$nodeParentHash{$nodeName}} = $nodeStatus;
		}
	}
} # (foreach)

# Do root node, areas
print "  /* Describing root, area nodes */\n";
$rootName="ROOT";
$rootStatus="5";
foreach $node (@nodes)
{
	chomp($node);
	@elements=split(/:/,$node);
	$nodeName=@elements[0];
	$nodeStatus=$parentStatus{$nodeName};
	$type=$shapeTypeHash{$nodeName};
	$shape=$shapeMapHash{$shapeTypeHash{$nodeName}};
	$color=$colorHash{$nodeStatus};
	if (@elements[1] eq "AREA")
	{
		$label=$nodeName;
		$label=~s/_/\\n/g;
		# Adjust root status if needed
		if (($statusHash{$rootStatus} > $statusHash{$nodeStatus})and($statusHash{$nodeStatus} ne "0"))
		{
			$rootStatus = $nodeStatus;
		}
		print "  $nodeName [label=\"$label\",shape=$shape,color=$color,style=filled,fontsize=$areaFont];\n";
	}

	if (@elements[1] eq "ROOT")
	{
		$rootName=$nodeName;
	}
} # (foreach)

# Describe root node
$color=$colorHash{$rootStatus};
$shape=$shapeMapHash{"ROOT"};
print "  $rootName [shape=$shape,color=$color,style=filled,fontsize=$rootFont,root];\n";

# Now describe all edges.
print "  /* Describing edges */\n";
foreach $node (@nodes)
{
	# Set up
	chomp($node);
	$startNode="";
	$endNode="";
	$type="";

	# Find end node
	@elements=split(/:/,$node);
	$endNode=@elements[0];

	# Find starting node (parent) and describe edge
	$type=$shapeTypeHash{$endNode};
	if ($type eq "OBJECTIVE")
	{
		$startNode=$nodeParentHash{$endNode};
		print "  $startNode -> $endNode ;\n";
		# [len=\"$objectiveLen\"];\n";
	} elsif ($type eq "AREA") {
		$startNode=$rootName;
		print "  $startNode -> $endNode ;\n";
		# [len=\"$areaLen\",weight=$areaWeight];\n";
	} # (what kind of deal?)
	
} # (foreach)

# Make graph footer
print "}\n";

sub showUsage {
	print "makeISOGraph.pl -- v1.0 -- Foofus.\n";
	print "  Usage: makeISOGraph.pl [options]\n";
	print "Available parameters:\n";
	print "  cm: colormap file (STATUS_CODE:color) - default is colormap.txt\n";
	print "  on: object name file (NodeName:TYPE) - defult is objectnames.txt\n";
	print "  sm: shape map file (TYPE:shape) - default is shapemap.txt\n";
	print "  rm: relationship file (NodeName:parent) - default is relationships.txt\n";
	print "  st: status file (ObjectiveNode:STATUS_CODE) - default is status.txt\n";
	print "  ga: any graph attributes you might want to specify - default is none\n";
	print "  af: font size for an area node (default is $areaFont)\n";
	print "  of: font size for an objective node (default is $objectiveFont)\n";
	print "  rf: font size for root node (default is $rootFont)\n";
	print "Output is .dot language, suitable for passing to circo or twopi\n";
} # (showUsage)
