#!/usr/bin/perl
#
#	mapAdjacencies.pl
#
#	Take a batch of pwdump/fgdump files and look for any recurring username
#	and password hash combinations.  On the basis of these, construct two
#	files: an adjacency list, and an adjacency matrix.  Maybe list some
#	interesting statistics, also.
#
#	This code is by Foofus (foofus at foofus dot net)
#

# Initialize default values
$version="0,1";
$mapdir="./maps";
$adjdir="./output/adjacency";
$indir="./input";
$suffix=".pwdump";
$prefix="map";
$exfile="./lists/exclude.list";

# Process command-line arguments
foreach $arg (@ARGV)
{
	chomp($arg);
	@pair=split(/:/,$arg);
	if (@pair[0] eq "in")
	{
		$indir=@pair[1];
	} elsif (@pair[0] eq "ad") {
		$adjdir=@pair[1];
	} elsif (@pair[0] eq "sf") {
		$suffix=@pair[1];
	} elsif (@pair[0] eq "pr") {
		$prefix=@pair[1];
	} elsif (@pair[0] eq "el") {
	        $exfile=@pair[1];
	} else {
		print "@pair[0] is not a supported option.\n";
		&showUsage;
		die;
	}
} # (foreach)

# Sanity check goes here
print "Welcome to mapAdjacencies.pl by Foofus.\n";
print "Input directory is $indir\n";
print "Adjacency files will be written to $adjdir\n";
print "These files will be identified by the prefix \"$prefix\"\n";
print "Excluding accounts beginning with strings listed in $exfile\n";

# Build the exclusion hash
open(INFILE,"<$exfile") || die "ERROR: Unable to open $exfile for reading!\n";
@excludeList=<INFILE>;
close(INFILE);

# Get a list of pwdump files
$numHosts=0;
opendir(DIR,$indir) || die ("ERROR: Unable to open input directory $indir\n");
while ($file=readdir(DIR))
{
        chomp($file);
        if (($file ne ".") and ($file ne "..") and (index($file,$suffix)>0))
        {
                push(@raw,$file);
		$numHosts++;
        } # (is this a pwdump file?)
	# Replace this with jmk's "sortIP"
        @files=sort(@raw);
} # (while)
closedir(DIR);
print "Total number of hosts: $numHosts\n";
if ($numHosts < 2) 
{ 
  die "There can be no adjacencies with only $numHosts hosts... Maybe you want a different input directory?\n";
}

# Walk through each file building hashes
$numAccounts=0;
$numTrust=0;
$numUnique=0;
%hostHosts=("hostName" => "neighborHosts");
%accountHosts=("accountHash" => "hostsConnected");
%hostAccounts=("hostName" => "accountsPresent" );
foreach $file (@files)
{
	chomp($file);

	# Extract the system name (ip)
	@pair=split(/$suffix/,$file);
	$hostName=@pair[0];

	# Read in the file
	open(INFILE,"<$indir/$file") || die ("ERROR: Unable to open file $indir/$file\n");
	@accounts=<INFILE>;
	close(INFILE);

	# Loop through the accounts
	foreach $account (@accounts)
	{
		# Extract the username and password, build a combined string
		@elements=split(/:/,$account);
		
		# If it's excluded, just plow on
		if (&isAllowed(@elements[0])==0) { 
			next; 
		}

		$accountString="@elements[0]-@elements[3]";
		$username=@elements[0];

		# Add this to the list of accounts on the host in question
		if (exists($hostAccounts{$hostName}))
		{
			$entry=$hostAccounts{$hostName};
			$entry=$entry.",$accountString";
			$hostAccounts{$hostName}=$entry;
		} else {
			$hostAccounts{$hostName}=$accountString;
		}

		# Have we seen this account before?
		if (exists($accountHosts{$accountString}))
		{
			# Append this host to that entry
			$entry=$accountHosts{$accountString};
			$entry=$entry.",$hostName";
			$accountHosts{$accountString}=$entry;

			# Increment the number of adjacencies
			$numTrust++;
		} else {
			# Add the account
			$accountHosts{$accountString}=$hostName;

			# Increment the number of unique accounts
			$numUnique++;
		}
		# Increment the total number of accounts
		$numAccounts++;
	} # (foreach)
} # (foreach)
print "Total number of account records processed: $numAccounts\n";
print "Number of unique username/password combinations: $numUnique\n";
print "Number of account recurrences (trust relationships) discovered: $numTrust\n";
print "Average number of local accounts per host: ".$numAccounts/$numHosts."\n";

# Build host-to-host adjacency list
%hostHosts=( "hostName" => "neighbors" );
foreach $host (keys(%hostAccounts))
{
	# Loop through the list of accounts on it
	$accounts=$hostAccounts{$host};
	@accountList=split(/,/,$accounts);

	# Look up what hosts that account is present on, add each to hash
	foreach $account (@accountList)
	{
		$affectedHosts=$accountHosts{$account};
		@neighborList=split(/,/,$affectedHosts);
		foreach $neighbor (@neighborList)
		{
			if ($host ne $neighbor)
			{
			if (exists($hostHosts{$host}))
			{
				# Append this neighbor to existing list if it's not already there
				if (index($hostHosts{$host},$neighbor)==-1)
				{
					$entry=$hostHosts{$host};
					$entry=$entry.",$neighbor";
					$hostHosts{$host}=$entry;
				}
			} else {
				# Create an entry
				$hostHosts{$host}=$neighbor;
			}
			}
		} # (foreach)
	} # (foreach)
} # (foreach)

# Build adjacency list files
# Hosts -> Accounts
open(OUTFILE,">$adjdir/$prefix-AccountsByHost.adj") || die "Could not open $adjdir/$prefix-AccountsByHost.adj for writing!\n";
foreach $host (keys(%hostAccounts))
{
	# Output only those hosts that are connected to some other host via an account
	if (exists($hostHosts{$host}))
	{
		print OUTFILE $host,":",$hostAccounts{$host},"\n";
	}
} # (foreach)
close(OUTFILE);

# Accounts -> Hosts
open(OUTFILE,">$adjdir/$prefix-HostsByAccount.adj") || die "Could not open $adjdir/$prefix-HostsByAccount for writing!\n";
foreach $account (keys(%accountHosts))
{
	#	Output only those accounts occuring on more than one host
	if (index($accountHosts{$account},",")>0)
	{
		print OUTFILE $account,":",$accountHosts{$account},"\n";
	}
}
close(OUTFILE);

# Hosts -> Hosts
open(OUTFILE,">$adjdir/$prefix-HostsByHost.adj") || die "Could not open $adjdir/$prefix-HostsByHost for writing!\n";
foreach $host (keys(%hostHosts))
{
	print OUTFILE $host,":",$hostHosts{$host},"\n";
}
close(OUTFILE);

# Host connected to the most neighbors
# Account present on the most hosts
# Host with most local accounts

###
### Subroutines begin here, now that the main program is all done
###
#	showUsage -- shows, um, usage
sub showUsage {
	print "mapAdjacencies.pl -- version $version by Foofus\n";
	print "This program builds lists of adjacencies in Windows password dump\n";
	print "files, and gathers some statistics.\n";
	print "usage: mapAdgacencies.pl [parameter:value] [parameter:value] [...]\n";
	print "Available parameters\n";
	print "in : the directory where input is stored (default is $indir)\n";
	print "ad : the directory where adjacency files will be written\n";
	print "     (default is $adjdir)\n";
	print "sf : suffix for pwdump/fgdump files (default is $suffix)\n";
	print "pr : prefix to prepend to output files (default is \"$prefix\")\n";
	print "el : file where account exclusion strings reside\n";
	print "     (default is $exfile) \n";
}	# (showUsage)

# isAllowed -- does a string match the exclusion list?
#              All hail, JMK!
sub isAllowed {
        local($name)=@_;
        foreach $item (@excludeList)
        {
    		if ($item =~ /^\/(.*)\/$/)
    		{
      			if ($name =~ /$1/)
      			{ 
				return(0);
			} 
    		} elsif ($name eq lc($item)) {
                        return(0);
                }
        } # (foreach)
        return(1);
} # (isAllowed)

